'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/* globals $ */

function append(msg, name) {
  $('#messages').append($('<li>').text(name + ': ' + msg));

  var height = $('#messages')[0].scrollHeight;
  $('#messages').animate({ scrollTop: height });
}

function rename(name, newName) {
  $('#messages > li').text(function (_, text) {
    return text.replace(name, newName);
  });
}

function clear() {
  $('#messages').html('');
}

function createChat(_ref) {
  var onCmd = _ref.onCmd,
      onMsg = _ref.onMsg,
      _ref$hidden = _ref.hidden,
      hidden = _ref$hidden === undefined ? false : _ref$hidden;

  if (hidden) {
    hide();
  }

  $('form').submit(function (event) {
    event.preventDefault();

    // parse whatever input the user submitted

    var _parseInput = parseInput($('#m').val()),
        chatMsg = _parseInput.chatMsg,
        cmd = _parseInput.cmd;

    $('#m').val('');

    if (chatMsg) {
      onMsg(chatMsg);
      // socket.emit('event', { type: '_msg', payload: chatMsg })
    } else if (cmd) {
      var args = cmd.split(' ');
      onCmd({
        type: args[0],
        args: args.slice(1)
      });
    }

    return false;
  });

  function parseInput(string) {
    var chatMsg = void 0;
    var cmd = void 0;

    if (string.substring(0, 1) === '/') {
      // treat as command
      cmd = string.substring(1);
    } else {
      chatMsg = string;
    }

    return { chatMsg: chatMsg, cmd: cmd };
  }

  function hide() {
    hidden = true;
    $('#chat-container').css('display', 'none');
    $(window).trigger('resize');
  }

  function show() {
    hidden = false;
    $('#chat-container').css('display', 'flex');
    $(window).trigger('resize');
  }

  function toggle() {
    if (hidden) {
      show();
    } else {
      hide();
    }
  }

  return {
    isHidden: function isHidden() {
      return hidden;
    },
    toggle: toggle,
    hide: hide,
    show: show,
    append: append,
    clear: clear,
    rename: rename
  };
}

exports.default = createChat;