'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _runStage = require('../run-stage');

var _runStage2 = _interopRequireDefault(_runStage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Enhances stage running with:
 * - html rendering
 * - canvas clearing
 */
function runOnClient(_ref) {
  var _ref$stage = _ref.stage;
  _ref$stage = _ref$stage === undefined ? {} : _ref$stage;
  var html = _ref$stage.html,
      setup = _ref$stage.setup,
      teardown = _ref$stage.teardown,
      _ref$stage$options = _ref$stage.options;
  _ref$stage$options = _ref$stage$options === undefined ? {} : _ref$stage$options;
  var duration = _ref$stage$options.duration,
      htmlContainerHeight = _ref$stage$options.htmlContainerHeight,
      _ref$context = _ref.context,
      context = _ref$context === undefined ? {} : _ref$context,
      timeout = _ref.timeout;

  var preHtmlContainerHeight = void 0;
  var modifiedSetup = function modifiedSetup() {
    preHtmlContainerHeight = context.getHtmlContainer().getHeightRatio();
    htmlContainerHeight && context.getHtmlContainer().setHeightRatio(htmlContainerHeight);

    html && context.renderHtml(html);
    setup && setup(context);
  };

  var modifiedTeardown = function modifiedTeardown() {
    var _context$getCanvas;

    context.getHtmlContainer().setHeightRatio(preHtmlContainerHeight);
    context.renderHtml('');

    (_context$getCanvas = context.getCanvas()).remove.apply(_context$getCanvas, _toConsumableArray(context.getCanvas().getObjects()));

    teardown && teardown(context);
  };

  return (0, _runStage2.default)({
    context: context,
    timeout: timeout,
    setup: modifiedSetup,
    teardown: modifiedTeardown,
    options: {
      duration: duration
    }
  });
}

/**
 * Manages stages on the client.
 */
function createManager(_ref2) {
  var _ref2$stages = _ref2.stages,
      stages = _ref2$stages === undefined ? [] : _ref2$stages,
      onStageFinished = _ref2.onStageFinished,
      getContext = _ref2.getContext;

  var events = {};
  var commands = {};

  var stopStage = void 0;
  var currentStage = -1;
  function startStage(stageNo) {
    if (stageNo >= stages.length) {
      return;
    }
    stopStage && stopStage();

    currentStage = stageNo;
    var stage = stages[currentStage];
    events = stage.events || {};
    commands = stage.commands || {};

    stopStage = runOnClient({
      stage: stage,
      context: getContext(),
      timeout: function timeout() {
        onStageFinished(currentStage);
      }
    });
  }

  function endStage(stageNo) {
    if (stageNo === currentStage) {
      stopStage && stopStage();
    }
    events = {};
    commands = {};
  }

  function stageFinished() {
    onStageFinished(currentStage);
  }

  return {
    startStage: startStage,
    endStage: endStage,
    stageFinished: stageFinished,

    getEvents: function getEvents() {
      return events;
    },
    getCommands: function getCommands() {
      return commands;
    }
  };
}

exports.default = createManager;