'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _chat = require('./chat.js');

var _chat2 = _interopRequireDefault(_chat);

var _canvas = require('./canvas.js');

var _canvas2 = _interopRequireDefault(_canvas);

var _htmlContainer = require('./html-container');

var _htmlContainer2 = _interopRequireDefault(_htmlContainer);

var _clientStageManager = require('./client-stage-manager');

var _clientStageManager2 = _interopRequireDefault(_clientStageManager);

var _socketClient = require('./socket-client');

var _socketClient2 = _interopRequireDefault(_socketClient);

var _isAdmin = require('./is-admin');

var _isAdmin2 = _interopRequireDefault(_isAdmin);

var _util = require('../util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var builtinEvents = {
  _msg: function _msg(monsterr, _ref) {
    var msg = _ref.msg,
        name = _ref.name;

    monsterr.getChat().append(msg, name);
  }
};

var builtinClientOnlyEvents = {
  _start_stage: function _start_stage(monsterr, stageNo) {
    monsterr.getStageManager().startStage(stageNo);
  },
  _end_stage: function _end_stage(monsterr, stageNo) {
    monsterr.getStageManager().endStage(stageNo);
  },
  _set_name: function _set_name(monsterr, _ref2) {
    var name = _ref2.name,
        prevName = _ref2.prevName;

    console.log(name, prevName);
    monsterr.getChat().rename(prevName, name);
  },
  _rename: function _rename(monsterr, _ref3) {
    var name = _ref3.name,
        prevName = _ref3.prevName;

    console.log(name, prevName);
    monsterr.getChat().rename(prevName, name);
  }
};

var builtinCommands = {
  clear: function clear(monsterr) {
    monsterr.getChat().clear();
    return false; // don't send this
  },
  id: function id(monsterr) {
    monsterr.getChat().append(monsterr.getId());
  },
  name: function name(monsterr, _name) {
    monsterr.send('_set_name', _name);
  }
};

function createClient() {
  var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref4$options = _ref4.options,
      options = _ref4$options === undefined ? {} : _ref4$options,
      _ref4$events = _ref4.events,
      events = _ref4$events === undefined ? {} : _ref4$events,
      _ref4$commands = _ref4.commands,
      commands = _ref4$commands === undefined ? {} : _ref4$commands,
      _ref4$stages = _ref4.stages,
      stages = _ref4$stages === undefined ? [] : _ref4$stages;

  var clientId = void 0;
  var socketClient = (0, _socketClient2.default)(_isAdmin2.default ? '/admin' : '/clients');

  stages = (0, _lodash.flattenDeep)(stages);

  function log(msg, fileOrExtra, extra) {
    socketClient.sendEvent({
      type: '_log',
      payload: { msg: msg, fileOrExtra: fileOrExtra, extra: extra }
    });
  }

  var chat = (0, _chat2.default)({
    onCmd: function onCmd(cmd) {
      var shouldCallServer = (0, _util.handleCommand)(cmd, [commands, builtinCommands, _isAdmin2.default ? {} : stageManager.getCommands()], monsterr);

      if (shouldCallServer) {
        socketClient.sendCommand(cmd);
      }
    },
    onMsg: function onMsg(msg) {
      return monsterr.send('_msg', msg);
    },
    hidden: options.hideChat || false
  });
  var htmlContainer = (0, _htmlContainer2.default)(options);
  var canvas = (0, _canvas2.default)(_extends({}, options, {
    getUsedWidth: function getUsedWidth() {
      return chat.isHidden() ? 0 : 300;
    },
    getUsedHeight: function getUsedHeight() {
      return htmlContainer.getHeightAbs();
    }
  }));

  var stageManager = !_isAdmin2.default ? (0, _clientStageManager2.default)({
    stages: stages,
    getContext: function getContext() {
      return monsterr;
    },
    onStageFinished: function onStageFinished(stageNo) {
      monsterr.send('_stage_finished', stageNo);
    }
  }) : undefined;

  socketClient.on('id', function (id) {
    clientId = id;
  });
  socketClient.on('disconnect', function () {
    return monsterr.disconnect();
  });
  socketClient.on('event', function (event) {
    (0, _util.handleEvent)(event, [events, builtinEvents, _isAdmin2.default ? {} : builtinClientOnlyEvents, _isAdmin2.default ? {} : stageManager.getEvents()], monsterr);
  });

  /** API */
  var monsterr = {
    send: function send(type, payload) {
      return socketClient.sendEvent({ type: type, payload: payload });
    },
    sendCommand: function sendCommand(type, args) {
      return socketClient.sendCommand({ type: type, args: args });
    },
    log: log,

    getId: function getId() {
      return clientId;
    },

    getChat: function getChat() {
      return chat;
    },
    getHtmlContainer: function getHtmlContainer() {
      return htmlContainer;
    },
    getCanvas: function getCanvas() {
      return canvas;
    },
    getStageManager: function getStageManager() {
      return stageManager;
    },

    getCommands: function getCommands() {
      return commands;
    },
    getEvents: function getEvents() {
      return events;
    },
    getStages: function getStages() {
      return stages;
    },


    renderHtml: function renderHtml(html) {
      return htmlContainer.render(html);
    },
    disconnect: function disconnect() {
      htmlContainer.setHeightRatio(1);
      htmlContainer.render('<h1>Disconnected...</h1>');
    },

    stageFinished: function stageFinished() {
      return stageManager.stageFinished();
    }
  };

  return monsterr;
}

exports.default = createClient;