'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _runStage = require('../run-stage');

var _runStage2 = _interopRequireDefault(_runStage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Manages stages on server.
 *
 * TODO: This is overloaded/complex and ripe for refactoring.
 * Given the event nature, this could be an EventEmitter.
 */
function createManager() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$getContext = _ref.getContext,
      getContext = _ref$getContext === undefined ? function () {
    return {};
  } : _ref$getContext,
      _ref$getPlayers = _ref.getPlayers,
      getPlayers = _ref$getPlayers === undefined ? function () {
    return [];
  } : _ref$getPlayers,
      _ref$onStageStarted = _ref.onStageStarted,
      onStageStarted = _ref$onStageStarted === undefined ? function () {} : _ref$onStageStarted,
      _ref$onStageEnded = _ref.onStageEnded,
      onStageEnded = _ref$onStageEnded === undefined ? function () {} : _ref$onStageEnded,
      _ref$onGameOver = _ref.onGameOver,
      onGameOver = _ref$onGameOver === undefined ? function () {} : _ref$onGameOver,
      _ref$stages = _ref.stages,
      stages = _ref$stages === undefined ? [] : _ref$stages;

  var started = false;
  var finishedPlayers = [];
  var currentStage = -1;
  var stopStage = void 0;

  var commands = {};
  var events = {};

  stages = (0, _lodash.flattenDeep)(stages);

  function checkIfDone() {
    var players = getPlayers().filter(function (p) {
      return p !== undefined;
    }).sort();

    if ((0, _lodash.difference)(players, finishedPlayers.sort()).length === 0) {
      finishedPlayers = [];
      nextStage();
    }
  }

  function playerFinishedStage(player, stageNo) {
    if (stageNo !== currentStage) {
      return;
    }

    finishedPlayers = (0, _lodash.uniq)([].concat(_toConsumableArray(finishedPlayers), [player]));
    checkIfDone();
  }

  function start() {
    if (currentStage === -1 && !started) {
      started = true;
      nextStage();
    }
  }

  function stopCurrentStage() {
    events = {};
    commands = {};

    if (stopStage) {
      stopStage();
    }
    if (currentStage >= 0 && currentStage < stages.length) {
      onStageEnded(currentStage);
    }
  }

  function nextStage() {
    if (!started || currentStage >= stages.length) {
      return;
    }

    stopCurrentStage();

    currentStage++;
    if (currentStage >= stages.length) {
      return onGameOver();
    }

    var stage = stages[currentStage];
    events = stage.events || {};
    commands = stage.commands || {};

    stopStage = (0, _runStage2.default)(_extends({}, stage, {
      context: getContext(),
      timeout: nextStage
    }));

    onStageStarted(currentStage);
  }

  function reset() {
    stopCurrentStage();
    currentStage = -1;
    started = false;
  }

  return {
    getCurrentStage: function getCurrentStage() {
      return currentStage;
    },
    getEvents: function getEvents() {
      return events;
    },
    getCommands: function getCommands() {
      return commands;
    },

    // helpers
    playerFinishedStage: playerFinishedStage,
    playerDisconnected: checkIfDone,

    // control stage flow
    start: start,
    next: nextStage,
    reset: reset
  };
}

exports.default = createManager;